#version 150 core
#extension GL_ARB_explicit_attrib_location : enable

layout(location = 0) out vec4 outColor0;
layout(location = 1) out vec4 outColor1;

uniform float MaterialIndex;
uniform vec3 color;

#if defined(TEXTURE) || defined(NORMALMAP)
uniform sampler2D texture0;
in vec2 uv0;
#endif

#ifdef NORMALMAP
uniform sampler2D NormalMap;
in vec3 tangent;
in vec3 bitangent;
#endif

in vec3 normal;

void main( void )
{
#ifdef COLOR
	outColor0 = vec4(color, MaterialIndex/255.0);
	outColor1 = vec4(normalize(normal.xyz), 0.0); // normals
#endif
#ifdef TEXTURE 
	outColor0 = vec4(texture(texture0, uv0).xyz * color, MaterialIndex/255.0);
	outColor1 = vec4(normalize(normal.xyz), 0.0); // normals
#endif
#ifdef NORMALMAP 
	mat3 tbn = mat3(tangent, bitangent,	normal);
	vec3 bumpNorm = tbn * normalize( (texture(NormalMap, uv0).rgb) * 2.0 - 1.0 );

	outColor0 = vec4(texture(texture0, uv0).xyz * color, MaterialIndex/255.0);
	outColor1 = vec4(normalize(bumpNorm.xyz), 0.0); // normals
#endif
}
